package cas.cordova.plugin;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.graphics.Color;
import android.os.Build;
import android.util.Log;
import android.view.DisplayCutout;
import android.view.Gravity;
import android.view.View;
import android.view.Window;
import android.view.WindowInsets;
import android.widget.FrameLayout;

import androidx.annotation.MainThread;
import androidx.annotation.NonNull;
import androidx.annotation.WorkerThread;

import com.cleversolutions.ads.AdCallback;
import com.cleversolutions.ads.AdError;
import com.cleversolutions.ads.AdSize;
import com.cleversolutions.ads.AdType;
import com.cleversolutions.ads.AdViewListener;
import com.cleversolutions.ads.LoadAdCallback;
import com.cleversolutions.ads.MediationManager;
import com.cleversolutions.ads.android.CASBannerView;
import com.cleveradssolutions.sdk.base.CASHandler;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

final class BannerWrapper {
    private static final int AD_POSITION_TOP_CENTER = 0;
    private static final int AD_POSITION_TOP_LEFT = 1;
    private static final int AD_POSITION_TOP_RIGHT = 2;
    private static final int AD_POSITION_BOTTOM_CENTER = 3;
    private static final int AD_POSITION_BOTTOM_LEFT = 4;
    private static final int AD_POSITION_BOTTOM_RIGHT = 5;

    @NotNull
    private final MediationManager manager;
    @NotNull
    private final Activity activity;

    private volatile int activePos = AD_POSITION_BOTTOM_CENTER;

    @Nullable
    public CASBannerView view;
    public AdSize activeSize = AdSize.BANNER;
    public boolean isNeedSafeInsets = true;

    private AdViewListener listener;
    private CASCBridge parent;

    public final Runnable showRunnable = () -> {
        if (view != null) {
            view.setVisibility(View.VISIBLE);
        } else {
            createBannerView();
        }
        view.setLayoutParams(getLayoutParams());
    };

    public final Runnable hideRunnable = () -> {
        if (view != null) {
            view.setVisibility(View.GONE);
        }
    };

    private final Runnable refreshPositionRunnable = () -> {
        if (view != null && view.getVisibility() == View.VISIBLE) {
            view.setLayoutParams(getLayoutParams());
        }
    };

    public BannerWrapper(@NotNull MediationManager manager, @NotNull Activity activity, @NotNull AdViewListener listener, @NotNull CASCBridge parent) {
        this.manager = manager;
        this.activity = activity;
        this.listener = listener;
        this.parent = parent;
    }

    @WorkerThread
    public final void loadNextAd() {
        if (view == null) {
            CASHandler.INSTANCE.main(() -> {
                createBannerView();
                view.setVisibility(View.GONE);
                view.loadNextAd();
            });
        } else {
            view.loadNextAd();
        }
    }

    @WorkerThread
    public final boolean setBannerSizeId(final int sizeId) {
        AdSize targetSize;
        switch (sizeId) {
            case 1:
                targetSize = AdSize.BANNER;
                break;
            case 2:
                targetSize = AdSize.getAdaptiveBannerInScreen(activity);
                break;
            case 3:
                targetSize = AdSize.getSmartBanner(activity);
                break;
            case 4:
                targetSize = AdSize.LEADERBOARD;
                break;
            case 5:
                targetSize = AdSize.MEDIUM_RECTANGLE;
                break;
            default:
                return false;
        }
        if (!targetSize.equals(activeSize)) {
            activeSize = targetSize;
            if (view != null) {
                view.setSize(activeSize);
            }
        }

        return true;
    }

    @WorkerThread
    public final void setBannerPositionId(final int value) {
        if (value < 0 || value > AD_POSITION_BOTTOM_RIGHT)
            activePos = AD_POSITION_BOTTOM_CENTER;
        else
            activePos = value;

        CASHandler.INSTANCE.main(refreshPositionRunnable);
    }

    @MainThread
    private void createBannerView() {
        view = new CASBannerView(activity, manager);
        view.setAdListener(listener);
        // Setting the background color works around an issue where the first ad isn't visible.
        view.setBackgroundColor(Color.TRANSPARENT);
        view.setGravity(Gravity.CENTER);
        activity.addContentView(view, getLayoutParams());
    }

    private FrameLayout.LayoutParams getLayoutParams() {
        final FrameLayout.LayoutParams adParams =
                new FrameLayout.LayoutParams(
                        FrameLayout.LayoutParams.WRAP_CONTENT, FrameLayout.LayoutParams.WRAP_CONTENT);
        final int targetPos = activePos;
        adParams.gravity = getLayoutGravityForPositionCode(targetPos);

        if (!isNeedSafeInsets || Build.VERSION.SDK_INT < Build.VERSION_CODES.P) {
            return adParams;
        }
        Window window = activity.getWindow();
        if (window == null) {
            return adParams;
        }
        WindowInsets windowInsets = window.getDecorView().getRootWindowInsets();
        if (windowInsets == null) {
            return adParams;
        }
        DisplayCutout displayCutout = windowInsets.getDisplayCutout();
        if (displayCutout == null) {
            return adParams;
        }

        adParams.bottomMargin = displayCutout.getSafeInsetBottom();
        adParams.rightMargin = displayCutout.getSafeInsetRight();
        adParams.leftMargin = displayCutout.getSafeInsetLeft();
        if (targetPos == AD_POSITION_TOP_CENTER
                || targetPos == AD_POSITION_TOP_LEFT
                || targetPos == AD_POSITION_TOP_RIGHT) {
            adParams.topMargin = displayCutout.getSafeInsetTop();
        }
        return adParams;
    }

    @SuppressLint("RtlHardcoded")
    private int getLayoutGravityForPositionCode(final int positionCode) {
        int gravity;
        switch (positionCode) {
            case AD_POSITION_TOP_CENTER:
                gravity = Gravity.TOP | Gravity.CENTER_HORIZONTAL;
                break;
            case AD_POSITION_TOP_LEFT:
                gravity = Gravity.TOP | Gravity.LEFT;
                break;
            case AD_POSITION_TOP_RIGHT:
                gravity = Gravity.TOP | Gravity.RIGHT;
                break;
            case AD_POSITION_BOTTOM_LEFT:
                gravity = Gravity.BOTTOM | Gravity.LEFT;
                break;
            case AD_POSITION_BOTTOM_RIGHT:
                gravity = Gravity.BOTTOM | Gravity.RIGHT;
                break;
            default:
                gravity = Gravity.BOTTOM | Gravity.CENTER_HORIZONTAL;
                break;
        }
        return gravity;
    }
}
